## Imports
import re, os, sys
import markdown, yaml
import extension

## Config
CONFIG_REGEX = r"---((.|\n)*?)---" # Regex used to find YAML config.
SECTIONS_REGEX = r"(?:^|\n)#\s([^\n]*)\n+(.*?)(?=\n#\s|$)" # Regex used to find each catagory - source: https://stackoverflow.com/a/66619938.
ARTICLES_SRC_DIR = "../articles_src" # Directory where the markdown files are stored.
ARTICLES_DIR = "../articles" # Directory where the generated files are saved.
CATAGORY_TO_STRING = { # Catagory strings as seen in the index.
	"Browser": "Web Browsers",
	"ContentPlatform": "Online Content Platforms",
	"Messaging": "Messaging Clients/Services",
	"SearchEngine": "Search Engines",
	"Games": "Video Games",
	"Media": "Media Players",
	"Misc": "Misc Software",
	"NotSpyware": "Not Spyware but Has Privacy Issues"
}

## Main
f = open("template_default.html", encoding="utf-8")
MAIN_HTML = f.read()
f.close()

md = markdown.Markdown(
	extensions=[
		extension.ArticleMarkdown() # The extension which adds various features to the markdown
	]
)

def convertMarkdown(inputStr, fileName):
	## Variable Setup
	# Get Markdown config
	configReg = re.match(CONFIG_REGEX, inputStr, re.MULTILINE)
	if configReg is None:
		raise Exception("Config regex failed")
	config = yaml.safe_load(configReg.group(1))

	# Get sections
	sections = {}
	sectionsReg = re.finditer(SECTIONS_REGEX, inputStr, re.DOTALL)
	if sectionsReg is None:
		raise Exception("Sections regex failed")
	for _, match in enumerate(sectionsReg, start=1):
		sections[match.group(1)] = match.group(2)

	## Document generation
	# Futher reading
	further = ""
	if "Further" in sections:
		further = """<hr/>
<h4>Further Reading:</h4>
{further}""".format(further = md.convert(sections['Further']))

	# Sources
	sources = ""
	if "Sources" in sections:
		a = ""
		for i in sections['Sources'].split("\n"):
			if i != "":
				result = re.match(r"(\d+)\. (.+)", i)
				a += """\n\t<li id="source{number}">{content}</li>""".format(
					number = result.group(1),
					content = md.convert(result.group(2))
				)

		sources = f"""<hr/>
<h4>Sources:</h4>
<ol>{a}
</ol>"""
	
	# Footer
	footer = "<hr/>"
	if "created" in config:
		footer += f"\n<b>This article was created on {config['created']}</b><br/>"
	if "modified" in config:
		footer += f"\n<b>This article was last modified on {config['modified']}</b><br/>"
	if "version" in config:
		footer += f"\n<b>This article is for version {config['version']} of {config['title']}</b><br/>"

	html = MAIN_HTML.format(
		title = config['title'],
		fileName = fileName,
		spywareLevel = config['rating'],
		introduction = md.convert(sections['Introduction']),
		content = md.convert(sections['Content']),
		further = further,
		sources = sources,
		footer = footer
	)

	return html, config

def convertFile(file):
	fileNoExt = file.split(".")[0]
	article, config = convertMarkdown(
		open(ARTICLES_SRC_DIR+"/"+file, encoding="utf8").read(),
		fileNoExt
	)
	f = open(ARTICLES_DIR+"/"+(fileNoExt+".html"), "w", encoding="utf8")
	f.write(article)
	f.close()

	return config

# Command line handler
if len(sys.argv) > 1:
	action = sys.argv[1]

	# Convert every file in ARTICLES_SRC_DIR
	if action == "convertAll":
		catagories = {}
		for file in os.listdir(ARTICLES_SRC_DIR):
			print(f"Converting {file}...")
			config = convertFile(file)
			if not config['catagory'] in catagories:
				catagories[config['catagory']] = {}
			catagories[config['catagory']][file.split(".")[0]] = config
		
		# Generate index.html
		f = open("template_index.html", encoding="utf-8")
		template = f.read()
		f.close()

		html = ""
		for catagoryName in catagories:
			if catagoryName != "Hidden": # Do not show hidden articles
				catagory = catagories[catagoryName]
				lHtml = ""
				for name in catagory:
					config = catagory[name]
					lHtml += f"""<li><a href="../articles/{name}.html">{config['title']}</a></li>"""
				html += f"""<div class="directory-list">
<h2>{CATAGORY_TO_STRING[catagoryName]}</h2>
<ul>
	{lHtml}
</ul>
</div>"""
		f = open(ARTICLES_DIR+"/index.html", "w", encoding="utf-8")
		f.write(template.format(
			articles = html
		))
		f.close()

	# Convert specific file in ARTICLES_SRC_DIR
	elif action == "convert":
		if len(sys.argv) >= 3:
			convertFile(sys.argv[2])
		else:
			print("You need to specify which file you want to convert.")
			exit()
	
	# Invalid action
	else:
		print("Unknown action.")
		exit()
	
	print("Task completed successfully")

else:
	# Usage
	print(f"""usage: convert.py [action] <args>

convert [fileName] - Convert specific file in {ARTICLES_SRC_DIR} and writes to {ARTICLES_DIR}
convertAll - Convert every file in {ARTICLES_SRC_DIR} and writes to {ARTICLES_DIR}""")
	exit()